@@ -327,6 +327,35 @@ do_post_sync_test (Test *test, gconstpointer data)
 
 }
 
+static void
+do_post_large_sync_test (Test *test, gconstpointer data)
+{
+        guint large_size = 1000000;
+        char *large_data;
+        unsigned int i;
+
+        large_data = g_malloc (large_size);
+        for (i = 0; i < large_size; i++)
+                large_data[i] = i & 0xFF;
+        GBytes *bytes = g_bytes_new_take (large_data, large_size);
+        test->msg = soup_message_new (SOUP_METHOD_POST, "https://127.0.0.1:5000/echo_post");
+        soup_message_set_request_body_from_bytes (test->msg, "text/plain", bytes);
+
+        GError *error = NULL;
+        GInputStream *response = soup_session_send (test->session, test->msg, NULL, &error);
+
+        g_assert_no_error (error);
+        g_assert_nonnull (response);
+
+        GBytes *response_bytes = read_stream_to_bytes_sync (response);
+        g_assert_true (g_bytes_equal (bytes, response_bytes));
+
+        g_bytes_unref (response_bytes);
+        g_object_unref (response);
+        g_bytes_unref (bytes);
+        g_object_unref (test->msg);
+}
+
 static void
 do_post_async_test (Test *test, gconstpointer data)
 {
@@ -354,6 +383,39 @@ do_post_async_test (Test *test, gconstpointer data)
         g_object_unref (test->msg);
 }
 
+static void
+do_post_large_async_test (Test *test, gconstpointer data)
+{
+        GMainContext *async_context = g_main_context_ref_thread_default ();
+        guint large_size = 1000000;
+        char *large_data;
+        unsigned int i;
+
+        large_data = g_malloc (large_size);
+        for (i = 0; i < large_size; i++)
+                large_data[i] = i & 0xFF;
+        GBytes *bytes = g_bytes_new_take (large_data, large_size);
+        test->msg = soup_message_new (SOUP_METHOD_POST, "https://127.0.0.1:5000/echo_post");
+        soup_message_set_request_body_from_bytes (test->msg, "text/plain", bytes);
+
+        GBytes *response = NULL;
+        soup_session_send_async (test->session, test->msg, G_PRIORITY_DEFAULT, NULL, on_send_complete, &response);
+
+        while (!response) {
+                g_main_context_iteration (async_context, TRUE);
+        }
+
+        g_assert_true (g_bytes_equal (bytes, response));
+
+        while (g_main_context_pending (async_context))
+                g_main_context_iteration (async_context, FALSE);
+
+        g_bytes_unref (response);
+        g_bytes_unref (bytes);
+        g_main_context_unref (async_context);
+        g_object_unref (test->msg);
+}
+
 static void
 do_post_blocked_async_test (Test *test, gconstpointer data)
 {
@@ -504,7 +566,7 @@ do_connections_test (Test *test, gconstpointer data)
 
 #define N_TESTS 100
 
-        for (uint i = 0; i < N_TESTS; ++i) {
+        for (unsigned int i = 0; i < N_TESTS; ++i) {
                 SoupMessage *msg = soup_message_new ("GET", "https://127.0.0.1:5000/slow");
                 soup_session_send_async (test->session, msg, G_PRIORITY_DEFAULT, NULL, on_send_ready, &complete_count);
                 g_object_unref (msg);
@@ -932,6 +994,25 @@ do_sniffer_sync_test (Test *test, gconstpointer data)
         do_one_sniffer_test (test->session, "https://127.0.0.1:5000/no-content", 0, FALSE, NULL);
 }
 
+static void
+do_timeout_test (Test *test, gconstpointer data)
+{
+        SoupMessage *msg;
+        GBytes *response;
+        GError *error = NULL;
+
+        soup_session_set_timeout (test->session, 2);
+
+        msg = soup_message_new (SOUP_METHOD_GET, "https://127.0.0.1:5000/timeout");
+        response = soup_test_session_async_send (test->session, msg, NULL, &error);
+        g_assert_null (response);
+        g_assert_error (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT);
+        g_object_unref (msg);
+
+        while (g_main_context_pending (NULL))
+                g_main_context_iteration (NULL, FALSE);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -976,6 +1057,14 @@ main (int argc, char **argv)
                     setup_session,
                     do_post_sync_test,
                     teardown_session);
+        g_test_add ("/http2/post/large/sync", Test, NULL,
+                    setup_session,
+                    do_post_large_sync_test,
+                    teardown_session);
+        g_test_add ("/http2/post/large/async", Test, NULL,
+                    setup_session,
+                    do_post_large_async_test,
+                    teardown_session);
         g_test_add ("/http2/post/blocked/async", Test, NULL,
                     setup_session,
                     do_post_blocked_async_test,
@@ -1032,6 +1121,10 @@ main (int argc, char **argv)
                     setup_session,
                     do_sniffer_sync_test,
                     teardown_session);
+        g_test_add ("/http2/timeout", Test, NULL,
+                    setup_session,
+                    do_timeout_test,
+                    teardown_session);
 
 	ret = g_test_run ();
 
